// Copyright (c) Mito
import React, { Fragment } from 'react';
import DefaultTaskpane from '../DefaultTaskpane';
import LargeSelect from '../../elements/LargeSelect';
import PivotTableKeySelection from './PivotTableKeySelection';
// Import 
import '../../../../css/pivot-taskpane.css';
import PivotTableValueSelection from './PivotTableValueSelection';
// NOTE: these aggregation functions need to be supported
// in mitosheet/steps/pivot.py as well
export var AggregationType;
(function (AggregationType) {
    AggregationType["SUM"] = "sum";
    AggregationType["MEAN"] = "mean";
    AggregationType["MEDIAN"] = "median";
    AggregationType["MIN"] = "min";
    AggregationType["MAX"] = "max";
    AggregationType["COUNT"] = "count";
    AggregationType["STD"] = "std";
    AggregationType["COUNT_UNIQUE"] = "count unique";
})(AggregationType || (AggregationType = {}));
export var KeyType;
(function (KeyType) {
    KeyType["Row"] = "Row";
    KeyType["Col"] = "Col";
})(KeyType || (KeyType = {}));
export var SelectionOperation;
(function (SelectionOperation) {
    SelectionOperation["ADD"] = "Add";
    SelectionOperation["REMOVE"] = "Remove";
})(SelectionOperation || (SelectionOperation = {}));
class PivotTaskpane extends React.Component {
    constructor(props) {
        super(props);
        this.editValueAggregationSelection = (columnHeader, operation, aggregationType) => {
            this.setState(prevState => {
                const newValues = JSON.parse(JSON.stringify(prevState.values));
                if (operation == SelectionOperation.ADD && aggregationType) {
                    // If the column is being added as a new aggregated value, add it
                    newValues[columnHeader] = aggregationType;
                }
                else if (operation == SelectionOperation.REMOVE) {
                    // If the column is being removed as an aggregated value, remove it
                    delete newValues[columnHeader];
                }
                return {
                    values: newValues
                };
            }, () => {
                void this.sendPivotTableUpdateMessage();
            });
        };
        /*
            A callback used by the select data source Select Element so that it can
            set the state of the Pivot Table Taskpane
        */
        this.setSelectedSheet = (newSheetName) => {
            const newSelectedSheetIndex = this.state.dfNames.indexOf(newSheetName);
            // If you didn't select a new sheet, then don't do clear your selections
            if (newSelectedSheetIndex == this.state.selectedSheetIndex) {
                return;
            }
            /*
                Reset the keys so
                    1) the column headers currently selected are dropped
                    2) the values currently aggregated are dropped
                    3) display the column headers from the new sheet
            */
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex,
                pivotRows: [],
                pivotCols: [],
                values: {}
            });
        };
        // Function used to add/remove column headers to the row and column keys 
        this.editKeySelection = (keyType, columnHeader, operation) => {
            // Sanity check: make sure that this column is not already selected. this should not be possible!
            if (operation === SelectionOperation.ADD) {
                if (this.state.pivotRows.includes(columnHeader) || this.state.pivotCols.includes(columnHeader)) {
                    console.log("column selected as key twice in pivot table. should not be possible");
                    return;
                }
            }
            this.setState(prevState => {
                let previousColumnHeaders = [];
                if (keyType === KeyType.Row) {
                    previousColumnHeaders = [...prevState.pivotRows];
                }
                else {
                    previousColumnHeaders = [...prevState.pivotCols];
                }
                if (operation == SelectionOperation.ADD && !previousColumnHeaders.includes(columnHeader)) {
                    // If the operation was add, add the column header to the list of column headers (if not already included)
                    previousColumnHeaders.push(columnHeader);
                }
                else if (operation == SelectionOperation.REMOVE && previousColumnHeaders.includes(columnHeader)) {
                    // If the operation was remove, add the column header to the list of column headers (if not already included)
                    previousColumnHeaders.splice(previousColumnHeaders.indexOf(columnHeader), 1);
                }
                return Object.assign(Object.assign({}, prevState), { [keyType === KeyType.Row ? 'pivotRows' : 'pivotCols']: previousColumnHeaders });
            }, () => {
                void this.sendPivotTableUpdateMessage();
            });
        };
        this.state = {
            dfNames: this.props.dfNames,
            selectedSheetIndex: props.selectedSheetIndex,
            pivotRows: [],
            pivotCols: [],
            values: {},
            keyError: '',
            stepID: ''
        };
        this.getSelectableKeyColumnHeaders = this.getSelectableKeyColumnHeaders.bind(this);
        this.editValueAggregationSelection = this.editValueAggregationSelection.bind(this);
        this.sendPivotTableUpdateMessage = this.sendPivotTableUpdateMessage.bind(this);
    }
    /*
        It is best practice to make async calls not in the constructor,
        but rather in the componentDidMount lifecycle method!

        Here, if there is anything to possibly pivot on, we send a pivot
        message.
    */
    componentDidMount() {
        // Send the first pivot message, if there is something to send
        if (this.props.sheetJSONArray.length > 0) {
            void this.sendPivotTableUpdateMessage();
        }
    }
    /*
        Completes the group operation by sending information for the group
        to the backend.
    */
    async sendPivotTableUpdateMessage() {
        const stepID = await this.props.mitoAPI.sendPivotMessage(this.state.selectedSheetIndex, this.state.pivotRows, this.state.pivotCols, this.state.values, this.state.stepID);
        this.setState({ stepID: stepID });
    }
    /*
        Returns the remaining possible column header keys.
        A column header can only be used in either the row or column key
    */
    getSelectableKeyColumnHeaders() {
        const allColumnHeaders = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns;
        return allColumnHeaders.filter(x => !this.state.pivotRows.includes(x) && !this.state.pivotCols.includes(x));
    }
    render() {
        /*
            If there is no possible Pivot taskpane that can be displayed (e.g. the sheetJSON is empty),
            give an error message indicating so.
        */
        if (this.props.sheetJSONArray.length === 0) {
            return (React.createElement(DefaultTaskpane, { header: 'Create a Pivot Table', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: React.createElement(Fragment, null, "Please import data before pivoting.") }));
        }
        const selectableColumnHeaders = this.getSelectableKeyColumnHeaders();
        const pivotRows = this.state.pivotRows;
        const pivotCols = this.state.pivotCols;
        return (React.createElement(DefaultTaskpane, { header: 'Create a Pivot Table', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: React.createElement(Fragment, null,
                React.createElement("div", { className: 'default-taskpane-body-section-div pivot-taskpane-section-header-div' },
                    React.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Data Source"),
                    React.createElement(LargeSelect, { startingValue: this.state.dfNames[this.state.selectedSheetIndex], key: this.state.selectedSheetIndex, optionsArray: this.state.dfNames, setValue: this.setSelectedSheet })),
                React.createElement("div", { className: 'default-taskpane-body-section-div' },
                    React.createElement(PivotTableKeySelection, { sectionTitle: 'Rows', selectableColumnHeaders: selectableColumnHeaders, selectedColumnHeaders: pivotRows, editKeySelection: this.editKeySelection, keyType: KeyType.Row })),
                React.createElement("div", { className: 'default-taskpane-body-section-div' },
                    React.createElement(PivotTableKeySelection, { sectionTitle: 'Columns', selectableColumnHeaders: selectableColumnHeaders, selectedColumnHeaders: pivotCols, editKeySelection: this.editKeySelection, keyType: KeyType.Col })),
                React.createElement("div", { className: 'default-taskpane-body-section-div' },
                    React.createElement(PivotTableValueSelection, { columnHeaders: this.props.sheetJSONArray[this.state.selectedSheetIndex].columns, aggregatedValuesInSection: this.state.values, editValueAggregationSelection: this.editValueAggregationSelection }))) }));
    }
}
export default PivotTaskpane;
//# sourceMappingURL=PivotTaskpane.js.map